@@ -17,7 +17,7 @@
 #
 """ Module for constants in Nuitka.
 
-This contains means to compare, classify and test constants.
+This contains tools to compare, classify and test constants.
 """
 
 import math
@@ -38,14 +38,15 @@ def compareConstants( a, b ):
     if type( a ) is not type( b ):
         return False
 
-    # Now it's either not the same, or it is a container that contains NaN or it is a
-    # complex or float that is NaN, the other cases can use == at the end.
+    # Now it's either not the same, or it is a container that contains NaN or it
+    # is a complex or float that is NaN, the other cases can use == at the end.
     if type( a ) is complex:
-        return compareConstants( a.imag, b.imag ) and compareConstants( a.real, b.real )
+        return compareConstants( a.imag, b.imag ) and \
+               compareConstants( a.real, b.real )
 
     if type( a ) is float:
-        # Check sign first, -0.0 is not 0.0, or -nan is not nan, it has a different sign
-        # for a start.
+        # Check sign first, -0.0 is not 0.0, or -nan is not nan, it has a
+        # different sign for a start.
         if math.copysign( 1.0, a ) != math.copysign( 1.0, b ):
             return False
 
@@ -70,7 +71,8 @@ def compareConstants( a, b ):
 
         for ea1, ea2 in iterItems( a ):
             for eb1, eb2 in iterItems( b ):
-                if compareConstants( ea1, eb1 ) and compareConstants( ea2, eb2 ):
+                if compareConstants( ea1, eb1 ) and \
+                   compareConstants( ea2, eb2 ):
                     break
             else:
                 return False
@@ -83,8 +85,8 @@ def compareConstants( a, b ):
 
         for ea in a:
             if ea not in b:
-                # Due to NaN values, we need to compare each set element with all the
-                # other set to be really sure.
+                # Due to NaN values, we need to compare each set element with
+                # all the other set to be really sure.
                 for eb in b:
                     if compareConstants( ea, eb ):
                         break
@@ -96,8 +98,8 @@ def compareConstants( a, b ):
     if type( a ) is range:
         return str( a ) == str( b )
 
-    # The NaN values of float and complex may let this fail, even if the constants are
-    # built in the same way.
+    # The NaN values of float and complex may let this fail, even if the
+    # constants are built in the same way.
     return a == b
 
 # These builtin type references are kind of constant too. TODO: The list is
@@ -137,8 +139,8 @@ def isConstant( constant ):
                 return False
         else:
             return True
-    elif constant_type in ( str, unicode, complex, int, long, bool, float, NoneType,
-                            range, bytes, set ):
+    elif constant_type in ( str, unicode, complex, int, long, bool, float,
+                            NoneType, range, bytes, set ):
         return True
     elif constant in ( Ellipsis, NoneType ):
         return True
@@ -147,11 +149,17 @@ def isConstant( constant ):
     else:
         return False
 
-
 def isMutable( constant ):
+    """ Is a constant mutable
+
+        That means a user of a reference to it, can modify it. Strings are
+        a prime example of mutable, dictionaries are mutable.
+    """
+
     constant_type = type( constant )
 
-    if constant_type in ( str, unicode, complex, int, long, bool, float, NoneType, range, bytes ):
+    if constant_type in ( str, unicode, complex, int, long, bool, float,
+                          NoneType, range, bytes ):
         return False
     elif constant_type in ( dict, list, set ):
         return True
@@ -162,16 +170,16 @@ def isMutable( constant ):
         else:
             return False
     elif constant is Ellipsis:
-        # Note: Workaround for Ellipsis not being handled by the pickle module,
-        # pretend it would be mutable, then it doesn't get pickled as part of lists or
-        # tuples. This is a loss of efficiency, but usage of Ellipsis will be very
-        # limited normally anyway.
+        return False
+    elif constant in constant_builtin_types:
         return True
     else:
         assert False, constant_type
 
 def isIterableConstant( constant ):
-    return type( constant ) in ( str, unicode, list, tuple, set, frozenset, dict, range, bytes )
+    return type( constant ) in (
+        str, unicode, list, tuple, set, frozenset, dict, range, bytes
+    )
 
 def getConstantIterationLength( constant ):
     assert isIterableConstant( constant )
@@ -208,3 +216,24 @@ class HashableConstant:
         assert isinstance( other, self.__class__ )
 
         return compareConstants( self.constant, other.constant )
+
+
+def createConstantDict( keys, values, lazy_order ):
+    if lazy_order:
+        constant_value = {}
+
+        keys = list(keys)
+        keys.reverse()
+
+        values = list(values)
+        values.reverse()
+    else:
+        constant_value = dict.fromkeys(
+            [ key for key in keys ],
+            None
+        )
+
+    for key, value in zip( keys, values ):
+        constant_value[ key ] = value
+
+    return constant_value
