@@ -29,6 +29,8 @@ from .tree import (
 
 from . import (
     ModuleRegistry,
+    SyntaxErrors,
+    Importing,
     Tracing,
     TreeXML,
     Options,
@@ -42,7 +44,17 @@ from .codegen import CodeGeneration
 from .optimizations import Optimization
 from .finalizations import Finalization
 
-import sys, os
+from nuitka.freezer.Standalone import (
+    detectEarlyImports,
+    detectPythonDLLs
+)
+from nuitka.freezer.BytecodeModuleFreezer import (
+    generateBytecodeFrozenCode,
+    getFrozenModuleCount,
+    addFrozenModule
+)
+
+import sys, os, subprocess, shutil
 
 from logging import warning
 
@@ -56,13 +68,27 @@ def createNodeTree( filename ):
     """
 
     # First, build the raw node tree from the source code.
-    result = Building.buildModuleTree(
+    main_module = Building.buildModuleTree(
         filename = filename,
         package  = None,
         is_top   = True,
         is_main  = not Options.shallMakeModule()
     )
-    ModuleRegistry.addRootModule( result )
+    ModuleRegistry.addRootModule( main_module )
+
+    # First remove old object files and old generated files, old binary or
+    # module, and standalone mode program directory if any, they can only do
+    # harm.
+    source_dir = getSourceDirectoryPath( main_module )
+    cleanSourceDirectory( source_dir )
+    if Options.isStandaloneMode():
+        standalone_dir = getStandaloneDirectoryPath( main_module )
+        shutil.rmtree( standalone_dir, ignore_errors = True )
+        Utils.makePath( standalone_dir )
+    Utils.deleteFile(
+        path       = getResultFullpath( main_module ),
+        must_exist = False
+    )
 
     # Second, do it for the directories given.
     for plugin_filename in Options.getShallFollowExtra():
@@ -74,7 +100,7 @@ def createNodeTree( filename ):
     # Then optimize the tree and potentially recursed modules.
     Optimization.optimize()
 
-    return result
+    return main_module
 
 def dumpTree( tree ):
     Tracing.printLine( "Analysis -> Tree Result" )
@@ -112,16 +138,45 @@ def getSourceDirectoryPath( main_module ):
         )
     )
 
-def getResultBasepath( main_module ):
-    assert main_module.isPythonModule()
-
+def getStandaloneDirectoryPath( main_module ):
     return Options.getOutputPath(
         path = Utils.basename(
-            getTreeFilenameWithSuffix( main_module, "" )
+            getTreeFilenameWithSuffix( main_module, ".dist" )
         )
     )
 
-def _cleanSourceDirectory( source_dir ):
+
+def getResultBasepath( main_module ):
+    assert main_module.isPythonModule()
+
+    if Options.isStandaloneMode():
+        return Utils.joinpath(
+            getStandaloneDirectoryPath( main_module ),
+            Utils.basename(
+                getTreeFilenameWithSuffix( main_module, "" )
+            )
+        )
+    else:
+        return Options.getOutputPath(
+            path = Utils.basename(
+                getTreeFilenameWithSuffix( main_module, "" )
+            )
+        )
+
+def getResultFullpath(main_module):
+    result = getResultBasepath(main_module)
+
+    if Options.shallMakeModule():
+        if Options.isWindowsTarget():
+            result += ".pyd"
+        else:
+            result += ".so"
+    else:
+        result += ".exe"
+
+    return result
+
+def cleanSourceDirectory( source_dir ):
     if Utils.isDir( source_dir ):
         for path, _filename in Utils.listDir( source_dir ):
             if Utils.getExtension( path ) in ( ".cpp", ".hpp", ".o", ".os" ):
@@ -136,7 +191,7 @@ def _cleanSourceDirectory( source_dir ):
             if Utils.getExtension( path ) in ( ".o", ".os" ):
                 Utils.deleteFile( path, True )
 
-def _pickSourceFilenames( source_dir, modules ):
+def pickSourceFilenames( source_dir, modules ):
     collision_filenames = set()
     seen_filenames = set()
 
@@ -167,7 +222,8 @@ def _pickSourceFilenames( source_dir, modules ):
         collision_filename = Utils.normcase( base_filename )
 
         if collision_filename in collision_filenames:
-            collision_counts[ collision_filename ] = collision_counts.get( collision_filename, 0 ) + 1
+            collision_counts[ collision_filename ] = \
+              collision_counts.get( collision_filename, 0 ) + 1
             hash_suffix = "@%d" % collision_counts[ collision_filename ]
         else:
             hash_suffix = ""
@@ -181,15 +237,14 @@ def _pickSourceFilenames( source_dir, modules ):
 
     return module_filenames
 
+standalone_entry_points = []
+
 def makeSourceDirectory( main_module ):
+    # We deal with a lot of details here, but rather one by one, and split makes
+    # no sense, pylint: disable=R0914
+
     assert main_module.isPythonModule()
 
-    source_dir = getSourceDirectoryPath( main_module )
-
-    # First remove old object files and old generated files, they can only do
-    # harm.
-    _cleanSourceDirectory( source_dir )
-
     # The global context used to generate code.
     global_context = CodeGeneration.makeGlobalContext()
 
@@ -198,12 +253,7 @@ def makeSourceDirectory( main_module ):
     assert main_module in modules
 
     # Sometimes we need to talk about all modules except main module.
-    other_modules = tuple(
-        m
-        for m in
-        modules
-        if not m is main_module and not m.isInternalModule()
-    )
+    other_modules = ModuleRegistry.getDoneUserModules()
 
     # Lets check if the recurse-to modules are actually present.
     for any_case_module in Options.getShallFollowModules():
@@ -218,10 +268,13 @@ def makeSourceDirectory( main_module ):
 
     # Prepare code generation, i.e. execute finalization for it.
     for module in sorted( modules, key = lambda x : x.getFullName() ):
-        Finalization.prepareCodeGeneration( module )
+        if module.isPythonModule():
+            Finalization.prepareCodeGeneration( module )
 
     # Pick filenames.
-    module_filenames = _pickSourceFilenames(
+    source_dir = getSourceDirectoryPath( main_module )
+
+    module_filenames = pickSourceFilenames(
         source_dir = source_dir,
         modules    = modules
     )
@@ -231,33 +284,59 @@ def makeSourceDirectory( main_module ):
     for module in sorted( modules, key = lambda x : x.getFullName() ):
         cpp_filename, hpp_filename = module_filenames[ module ]
 
-        source_code, header_code, module_context = \
-          CodeGeneration.generateModuleCode(
-            global_context = global_context,
-            module         = module,
-            module_name    = module.getFullName(),
-            other_modules  = other_modules if module is main_module else ()
-        )
-
-        # The main of an executable module gets a bit different code.
-        if module is main_module and not Options.shallMakeModule():
-            source_code = CodeGeneration.generateMainCode(
-                context = module_context,
-                module  = module,
-                codes   = source_code
+        if module.isPythonModule():
+            source_code, header_code, module_context = \
+              CodeGeneration.generateModuleCode(
+                  global_context = global_context,
+                  module         = module,
+                  module_name    = module.getFullName(),
+                  other_modules  = other_modules
+                                     if module is main_module else
+                                   ()
             )
 
-        module_hpps.append( hpp_filename )
+            # The main of an executable module gets a bit different code.
+            if module is main_module and not Options.shallMakeModule():
+                source_code = CodeGeneration.generateMainCode(
+                    context = module_context,
+                    codes   = source_code
+                )
 
-        writeSourceCode(
-            filename     = cpp_filename,
-            source_code  = source_code
-        )
+            module_hpps.append( hpp_filename )
 
-        writeSourceCode(
-            filename     = hpp_filename,
-            source_code  = header_code
-        )
+            writeSourceCode(
+                filename     = cpp_filename,
+                source_code  = source_code
+            )
+
+            writeSourceCode(
+                filename     = hpp_filename,
+                source_code  = header_code
+            )
+        elif module.isPythonShlibModule():
+            target_filename = Utils.joinpath(
+                getStandaloneDirectoryPath( main_module ),
+                *module.getFullName().split( "." )
+            )
+
+            if Options.isWindowsTarget():
+                target_filename += ".pyd"
+            else:
+                target_filename += ".so"
+
+            target_dir = Utils.dirname( target_filename )
+
+            if not Utils.isDir( target_dir ):
+                Utils.makePath( target_dir )
+
+            shutil.copy(
+                module.getFilename(),
+                target_filename
+            )
+
+            standalone_entry_points.append( target_filename )
+        else:
+            assert False, module
 
     writeSourceCode(
         filename    = Utils.joinpath( source_dir, "__constants.hpp" ),
@@ -297,6 +376,9 @@ def makeSourceDirectory( main_module ):
     )
 
 def runScons( main_module, quiet ):
+    # Scons gets transported many details, that we express as variables, and
+    # have checks for them, leading to many branches, pylint: disable=R0912
+
     python_version = "%d.%d" % ( sys.version_info[0], sys.version_info[1] )
 
     if hasattr( sys, "abiflags" ):
@@ -329,12 +411,17 @@ def runScons( main_module, quiet ):
         "python_version" : python_version,
         "target_arch"    : Utils.getArchitecture(),
         "python_prefix"  : sys.prefix,
-        "nuitka_src"     : Utils.joinpath(
-            SconsInterface.getSconsDataPath(),
-            "static_src"
+        "nuitka_src"     : SconsInterface.getSconsDataPath(),
+        "module_count"   : "%d" % (
+            len( ModuleRegistry.getDoneUserModules() ) + 1
         )
     }
 
+    # Ask Scons to cache on Windows, except where the directory is thrown
+    # away. On non-Windows you can should use ccache instead.
+    if not Options.isRemoveBuildDir() and os.name == "nt":
+        options[ "cache_mode" ] = "true"
+
     if Options.isLto():
         options[ "lto_mode" ] = "true"
 
@@ -344,8 +431,13 @@ def runScons( main_module, quiet ):
     if Options.shallDisableConsoleWindow():
         options[ "win_disable_console" ] = "true"
 
-    if Options.isPortableMode():
-        options[ "portable_mode" ] = "true"
+    if Options.isStandaloneMode():
+        options[ "standalone_mode" ] = "true"
+
+    if getFrozenModuleCount():
+        options[ "frozen_modules" ] = str(
+            getFrozenModuleCount()
+        )
 
     if Options.isShowScons():
         options[ "show_scons" ] = "true"
@@ -393,18 +485,12 @@ def callExec( args, clean_path, add_path ):
     args += Options.getMainArgs()
 
     # That's the API of execl, pylint: disable=W0142
-    os.execl( *args )
-
-    if old_python_path is not None:
-        os.environ[ "PYTHONPATH" ] = old_python_path
-    else:
-        del os.environ[ "PYTHONPATH" ]
-
+    Utils.callExec( args )
 
 def executeMain( binary_filename, tree, clean_path ):
     main_filename = tree.getFilename()
 
-    if Options.isPortableMode():
+    if Options.isStandaloneMode():
         name = binary_filename
     elif main_filename.endswith( ".py" ):
         name = main_filename[:-3]
@@ -413,10 +499,10 @@ def executeMain( binary_filename, tree, clean_path ):
 
     name = os.path.abspath( name )
 
-    if not Options.isWindowsTarget() or os.name == "nt":
-        args = ( binary_filename, name )
-    else:
+    if Options.isWindowsTarget() and os.name != "nt":
         args = ( "/usr/bin/wine", "wine", binary_filename )
+    else:
+        args = ( binary_filename, name )
 
     callExec(
         clean_path = clean_path,
@@ -444,11 +530,28 @@ def executeModule( tree, clean_path ):
     )
 
 def compileTree( main_module ):
+    source_dir = getSourceDirectoryPath( main_module )
+
     if not Options.shallOnlyExecGcc():
         # Now build the target language code for the whole tree.
         makeSourceDirectory(
             main_module = main_module
         )
+
+        if Options.isStandaloneMode():
+            for early_import in detectEarlyImports():
+                addFrozenModule( early_import )
+
+        if getFrozenModuleCount():
+            frozen_code = generateBytecodeFrozenCode()
+
+            writeSourceCode(
+                filename = Utils.joinpath(
+                    source_dir,
+                    "__frozen.cpp"
+                ),
+                source_code = frozen_code
+            )
     else:
         source_dir = getSourceDirectoryPath( main_module )
 
@@ -463,3 +566,120 @@ def compileTree( main_module ):
     )
 
     return result, options
+
+def main():
+    """ Main program flow of Nuitka
+
+        At this point, options will be parsed already, Nuitka will be executing
+        in the desired version of Python with desired flags, and we just get
+        to execute the task assigned.
+
+        We might be asked to only re-compile generated C++, dump only an XML
+        representation of the internal node tree after optimization, etc.
+    """
+
+    # Main has to fullfil many options, leading to many branches
+    # pylint: disable=R0912
+
+    positional_args = Options.getPositionalArgs()
+    assert len( positional_args ) > 0
+
+    filename = Options.getPositionalArgs()[0]
+
+    # Inform the importing layer about the main script directory, so it can use
+    # it when attempting to follow imports.
+    Importing.setMainScriptDirectory(
+        main_dir = os.path.dirname( os.path.abspath( filename ) )
+    )
+
+    # Turn that source code into a node tree structure.
+    try:
+        main_module = createNodeTree(
+            filename = filename
+        )
+    except (SyntaxError, IndentationError) as e:
+        if Options.isFullCompat() and \
+           e.args[0].startswith( "unknown encoding:" ):
+            if Utils.python_version >= 333 or \
+               (
+                   Utils.python_version >= 276 and \
+                   Utils.python_version < 300
+               ) or \
+               "2.7.5+" in sys.version or \
+               "3.3.2+" in sys.version: # Debian backports have "+" versions
+                complaint = "no-exist"
+            else:
+                complaint = "with BOM"
+
+            e.args = (
+                "encoding problem: %s" % complaint,
+                ( e.args[1][0], 1, None, None )
+            )
+
+        sys.exit( SyntaxErrors.formatOutput( e ) )
+
+    if Options.shallDumpBuiltTree():
+        dumpTree( main_module )
+    elif Options.shallDumpBuiltTreeXML():
+        dumpTreeXML( main_module )
+    elif Options.shallDisplayBuiltTree():
+        displayTree( main_module )
+    else:
+        result, options = compileTree( main_module )
+
+        # Exit if compilation failed.
+        if not result:
+            sys.exit( 1 )
+
+        # Remove the source directory (now build directory too) if asked to.
+        if Options.isRemoveBuildDir():
+            shutil.rmtree( getSourceDirectoryPath( main_module ) )
+
+        if Options.isStandaloneMode():
+            binary_filename = options[ "result_name" ] + ".exe"
+
+            standalone_entry_points.append( binary_filename )
+
+            for early_dll in detectPythonDLLs( standalone_entry_points ):
+                shutil.copy(
+                    early_dll,
+                    Utils.joinpath(
+                        getStandaloneDirectoryPath( main_module ),
+                        Utils.basename( early_dll )
+                    )
+                )
+
+        # Sanity check, warn people if "__main__" is used in the compiled
+        # module, it may not be the appropiate usage.
+        if Options.shallMakeModule() and Options.shallExecuteImmediately():
+            for variable in main_module.getVariables():
+                if variable.getName() == "__name__":
+                    warning( """\
+Compiling to extension module, which will not have '__name__' as '__main__', \
+did you intend '--exe' or to use 'nuitka-python' instead.""" )
+                    break
+
+        # Modules should not be executable, but Scons creates them like it, fix
+        # it up here.
+        if os.name != "nt" and Options.shallMakeModule():
+            subprocess.call(
+                (
+                    "chmod",
+                    "-x",
+                    getResultFullpath( main_module )
+                )
+            )
+
+        # Execute the module immediately if option was given.
+        if Options.shallExecuteImmediately():
+            if Options.shallMakeModule():
+                executeModule(
+                    tree       = main_module,
+                    clean_path = Options.shallClearPythonPathEnvironment()
+                )
+            else:
+                executeMain(
+                    binary_filename = getResultFullpath( main_module ),
+                    tree            = main_module,
+                    clean_path      = Options.shallClearPythonPathEnvironment()
+                )
