@@ -23,11 +23,13 @@ from nuitka.nodes.ParameterSpecs import ParameterSpec
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTargetTempVariableRef,
     ExpressionTempVariableRef,
-    ExpressionVariableRef,
-    StatementTempBlock
+    ExpressionVariableRef
 )
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
-from nuitka.nodes.AssignNodes import StatementAssignmentVariable
+from nuitka.nodes.AssignNodes import (
+    StatementAssignmentVariable,
+    StatementDelVariable
+)
 from nuitka.nodes.StatementNodes import (
     StatementExpressionOnly,
     StatementsSequence,
@@ -88,7 +90,8 @@ def buildListContractionNode( provider, node, source_ref ):
             constant   = [],
             source_ref = source_ref
         ),
-        # Note: For Python3, the list contractions no longer assign to the outer scope.
+        # Note: For Python3, the list contractions no longer assign to the outer
+        # scope.
         assign_provider  = Utils.python_version < 300,
         source_ref       = source_ref
     )
@@ -140,15 +143,16 @@ def buildGeneratorExpressionNode( provider, node, source_ref ):
         source_ref       = source_ref
     )
 
-def _buildContractionNode( provider, node, name, emit_class, start_value, assign_provider, source_ref ):
-    # The contraction nodes are reformulated to function bodies, with loops as described
-    # in the developer manual. They use a lot of temporary names, nested blocks, etc. and
-    # so a lot of variable names. There is no good way around that, and we deal with many
-    # cases, due to having generator expressions sharing this code,
-    # pylint: disable=R0912,R0914
+def _buildContractionNode( provider, node, name, emit_class, start_value,
+                           assign_provider, source_ref ):
+    # The contraction nodes are reformulated to function bodies, with loops as
+    # described in the developer manual. They use a lot of temporary names,
+    # nested blocks, etc. and so a lot of variable names. There is no good way
+    # around that, and we deal with many cases, due to having generator
+    # expressions sharing this code, pylint: disable=R0912,R0914
 
-    # Note: The assign_provider is only to cover Python2 list contractions, assigning one
-    # of the loop variables to the outside scope.
+    # Note: The assign_provider is only to cover Python2 list contractions,
+    # assigning one of the loop variables to the outside scope.
 
     assert provider.isParentVariableProvider(), provider
 
@@ -160,17 +164,13 @@ def _buildContractionNode( provider, node, name, emit_class, start_value, assign
         source_ref = source_ref
     )
 
-    temp_block = StatementTempBlock(
-        source_ref = source_ref
-    )
-
     if start_value is not None:
-        container_tmp = temp_block.getTempVariable( "result" )
+        container_tmp = function_body.allocateTempVariable( None, "result" )
 
         statements = [
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = container_tmp.makeReference( temp_block ),
+                    variable   = container_tmp.makeReference( function_body ),
                     source_ref = source_ref
                 ),
                 source     = start_value,
@@ -184,7 +184,7 @@ def _buildContractionNode( provider, node, name, emit_class, start_value, assign
         if start_value is not None:
             current_body = emit_class(
                 ExpressionTempVariableRef(
-                    variable   = container_tmp.makeReference( temp_block ),
+                    variable   = container_tmp.makeReference( function_body ),
                     source_ref = source_ref
                 ),
                 buildNode(
@@ -212,7 +212,7 @@ def _buildContractionNode( provider, node, name, emit_class, start_value, assign
 
         current_body = emit_class(
             ExpressionTempVariableRef(
-                variable   = container_tmp.makeReference( temp_block ),
+                variable   = container_tmp.makeReference( function_body ),
                 source_ref = source_ref
             ),
             key = buildNode(
@@ -233,17 +233,20 @@ def _buildContractionNode( provider, node, name, emit_class, start_value, assign
         source_ref = source_ref
     )
 
-    for qual in reversed( node.generators ):
-        nested_temp_block = StatementTempBlock(
-            source_ref = source_ref
+    for count, qual in enumerate( reversed( node.generators ) ):
+        tmp_iter_variable = function_body.allocateTempVariable(
+            temp_scope = None,
+            name       = "contraction_iter_%d" % count
         )
 
-        tmp_iter_variable = nested_temp_block.getTempVariable( "contraction_iter" )
+        tmp_value_variable = function_body.allocateTempVariable(
+            temp_scope = None,
+            name       = "iter_value_%d" % count
+        )
 
-        tmp_value_variable = nested_temp_block.getTempVariable( "iter_value" )
-
-        # The first iterated value is to be calculated outside of the function and
-        # will be given as a parameter "_iterated".
+        # The first iterated value is to be calculated outside of the function
+        # and will be given as a parameter "_iterated", the others are built
+        # inside the function.
         if qual is node.generators[0]:
             value_iterator = ExpressionVariableRef(
                 variable_name = "__iterator",
@@ -263,7 +266,9 @@ def _buildContractionNode( provider, node, name, emit_class, start_value, assign
         nested_statements = [
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_iter_variable.makeReference( nested_temp_block ),
+                    variable   = tmp_iter_variable.makeReference(
+                        function_body
+                    ),
                     source_ref = source_ref
                 ),
                 source     = value_iterator,
@@ -276,12 +281,16 @@ def _buildContractionNode( provider, node, name, emit_class, start_value, assign
                 tried          = makeStatementsSequenceFromStatement(
                     statement = StatementAssignmentVariable(
                         variable_ref = ExpressionTargetTempVariableRef(
-                            variable   = tmp_value_variable.makeReference( nested_temp_block ),
+                            variable   = tmp_value_variable.makeReference(
+                                function_body
+                            ),
                             source_ref = source_ref
                         ),
                         source     = ExpressionBuiltinNext1(
                             value      = ExpressionTempVariableRef(
-                                variable   = tmp_iter_variable.makeReference( nested_temp_block ),
+                                variable   = tmp_iter_variable.makeReference(
+                                    function_body
+                                ),
                                 source_ref = source_ref
                             ),
                             source_ref = source_ref
@@ -298,13 +307,16 @@ def _buildContractionNode( provider, node, name, emit_class, start_value, assign
                 source_ref     = source_ref
             ),
             buildAssignmentStatements(
-                provider   = provider if assign_provider else function_body,
-                node       = qual.target,
-                source     = ExpressionTempVariableRef(
-                    variable   = tmp_value_variable.makeReference( nested_temp_block ),
+                provider      = provider if assign_provider else function_body,
+                temp_provider = function_body,
+                node          = qual.target,
+                source        = ExpressionTempVariableRef(
+                    variable   = tmp_value_variable.makeReference(
+                        function_body
+                    ),
                     source_ref = source_ref
                 ),
-                source_ref = source_ref
+                source_ref    = source_ref
             )
         ]
 
@@ -353,14 +365,23 @@ def _buildContractionNode( provider, node, name, emit_class, start_value, assign
             )
         )
 
-        nested_temp_block.setBody(
-            StatementsSequence(
-                statements = nested_statements,
+        nested_statements.append(
+            StatementDelVariable(
+                variable_ref = ExpressionTargetTempVariableRef(
+                    variable   = tmp_iter_variable.makeReference(
+                        function_body
+                    ),
+                    source_ref = source_ref
+                ),
+                tolerant   = False,
                 source_ref = source_ref
             )
         )
 
-        current_body = nested_temp_block
+        current_body = StatementsSequence(
+            statements = nested_statements,
+            source_ref = source_ref
+        )
 
     statements.append( current_body )
 
@@ -368,23 +389,16 @@ def _buildContractionNode( provider, node, name, emit_class, start_value, assign
         statements.append(
             StatementReturn(
                 expression = ExpressionTempVariableRef(
-                    variable   = container_tmp.makeReference( temp_block ),
+                    variable   = container_tmp.makeReference( function_body ),
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
             )
         )
 
-    temp_block.setBody(
-        StatementsSequence(
-            statements = statements,
-            source_ref = source_ref
-        )
-    )
-
     function_body.setBody(
         StatementsFrame(
-            statements    = [ temp_block ],
+            statements    = statements,
             guard_mode    = "pass_through" if emit_class is not ExpressionYield else "generator",
             var_names     = (),
             arg_count     = 0,
